*****************************************************************************
* Program  : sv.prg
* Language : Clipper summer '87
* Notes    : Support functions for accessing script variables created by XIVR
* Copyright: Xtend Technologies (P) Ltd.  grants  all legal XIVR users
*          : the right to use, modify or include the following source
*          : code in your programs.
*****************************************************************************
*  Functions
*  ---------
*  svOpen       Opens the parameter file passed by XIVR demon as
*               the command line argument
*  svGetString  Returns the value of a string variable from the
*               parameter file
*  svSetString  Writes the value of the string variable into the
*               parameter file
*  svGetNum     Returns the value of a numeric variable from the
*               parameter file
*  svSetNum     Writes the value of a numeric variable into the
*               parameter file
*  svClose      Close and updates the parameter file. Commit all
*               write commands.
*****************************************************************************
* Limitations
*  1. This source code is hard coded for handling 200 script variables.
*     If you want to use more than 200 script variables, you require
*     changes in this code. Replace all '200' with the desired number.
*
*  2. An array "X_VAR" is used to keep all the private variables in it,
*     to avoid the conflict between variable names in other program
*     modules.
*
*     The variables used in this code are ,
*
*     X_VAR, pf_VAR, pf_VAL     ( Arrays )
*     X_Buffer, X_File, , X_StrVar, X_NumVar, X_NumVal  (Variables)
*
*     X_VAR[1] = File Handle obtained from "fopen" function.
*     X_VAR[2] = A counter variable for "DO WHILE" loop.
*     X_VAR[3] = A Flag to check EOF of the flat file.
*     X_VAR[4] = Number of bytes returned from "fread" function.
*     X_VAR[5] = Not used.
*     X_VAR[6] = The size of the array which holds the Script Variables
*     X_VAR[7] = A String accumulator
*     X_VAR[8] = The array subscript
*     X_VAR[9] = Name of the parameter file
*****************************************************************************

* ---------------
*  Function     : svOpen()
*  Parameters   : Name of the file containing the script variables
*  Return Value : .T. if successful, .F. on failure
*  Notes        : You should call this function, before calling any other
*               : functions.
* ---------------
Function svOpen
param X_File

if left(type("X_VAR[1]"),1) != 'U'
   return .F.         && The svOpen function is called more than once !
endif

* Hold all script variables and their values in pf_VAR and pf_VAL arrays
public pf_VAR[200],pf_VAL[200]

* Hold all local variables in X_VAR array to avoid variable name conflict
public X_VAR[9]

* Open the parameter file
* -----------------------
X_VAR[1] = fopen(X_File,0)
if X_VAR[1] = -1
   release X_VAR
   release pf_VAR
   release pf_VAL
   return .F.
endif

X_VAR[2] = 1              && Counter Variable
X_VAR[3] = .F.            && EOF Flag
X_VAR[7] = ''             && String Accumulator
X_VAR[8] = 0              && Array Subscript
X_VAR[9] = X_File         && Name of the parameter file
X_Buffer = ' '            && Read Buffer (1 Byte)

do while ! X_VAR[3]
   X_VAR[4] = fread(X_VAR[1],@X_Buffer,1)
   if X_VAR[4] != 1
      X_VAR[3] = .T.
      loop
   endif

   if X_Buffer != chr(13) .and. X_Buffer != chr(10)
      X_VAR[7] = X_VAR[7] + X_Buffer
      loop
   endif

   X_VAR[7] = alltrim(X_VAR[7])
   if len(X_VAR[7]) == 0
      loop
   endif
   X_VAR[7] = Hex2Text(X_VAR[7])   && Coverting hexadecimal numbers to ASCII values

   X_VAR[2] = 1
   do while X_VAR[2] <= len(X_VAR[7])
      if substr(X_VAR[7],X_VAR[2],1) = ' '
	 X_VAR[8] = X_VAR[8] + 1
	 pf_VAR[X_VAR[8]] = left(X_VAR[7], X_VAR[2])
	 pf_VAL[X_VAR[8]] = substr(X_VAR[7], X_VAR[2], len(X_VAR[7]))
	 pf_VAR[X_VAR[8]] = alltrim(pf_VAR[X_VAR[8]])
	 pf_VAL[X_VAR[8]] = alltrim(pf_VAL[X_VAR[8]])
	 if X_VAR[8] >= 200
	    fclose(X_VAR[1])
	    release X_VAR
	    release pf_VAR
	    release pf_VAL
	    return .F.
	 endif
	 exit
      endif
      X_VAR[2] = X_VAR[2] + 1
   enddo
   X_VAR[7] = ''
enddo

X_VAR[7] = alltrim(X_VAR[7])

if len(X_VAR[7]) > 0    && If \r \n is missing in the last line,

   X_VAR[2] = 1         && then control comes here
   do while X_VAR[2] <= len(X_VAR[7])
      if substr(X_VAR[7],X_VAR[2],1) = ' '
	 X_VAR[8] = X_VAR[8] + 1
	 pf_VAR[X_VAR[8]] = left(X_VAR[7], X_VAR[2])
	 pf_VAL[X_VAR[8]] = substr(X_VAR[7], X_VAR[2], len(X_VAR[7]))
	 pf_VAR[X_VAR[8]] = alltrim(pf_VAR[X_VAR[8]])
	 pf_VAL[X_VAR[8]] = alltrim(pf_VAL[X_VAR[8]])
	 if X_VAR[8] >= 200
	    fclose(X_VAR[1])
	    release X_VAR
	    release pf_VAR
	    release pf_VAL
	    return .F.
	 endif
	 exit
      endif
      X_VAR[2] = X_VAR[2] + 1
   enddo
   X_VAR[7] = ''
endif
fclose(X_VAR[1])
return .T.

* ---------------
*  Function     : svClose()
*  Parameters   : No parameters
*  Return Value : .T. if successful, .F. on failure
*  Notes        : Call this function before exiting from your application.
*               : Otherwise, the new values written into variables would not
*               : get modified.
* ---------------

Function svClose

if type("X_VAR[1]") == 'U'
   return .F.         && The svOpen function is not called  !
endif

* Re-create the parameter file with the new Variables and values
X_VAR[1] = fcreate(X_VAR[9],0)
if X_VAR[1] == -1
   release X_VAR
   release pf_VAR
   release pf_VAL
   return .F.
endif
X_VAR[2] = 1
do while X_VAR[2] <= X_VAR[8]
   X_VAR[7] = pf_VAR[X_VAR[2]]
   X_VAR[7] = X_VAR[7] + ' '
   X_VAR[7] = X_VAR[7] + pf_VAL[X_VAR[2]]
   X_VAR[7] = Text2Hex(X_VAR[7]) + chr(13) + chr(10)

   if fwrite(X_VAR[1],X_VAR[7],len(X_VAR[7])) != len(X_VAR[7])
      fclose(X_VAR[1])
      release X_VAR
      release pf_VAR
      release pf_VAL
      return .F.
   endif
   X_VAR[2] = X_VAR[2] + 1
enddo
if ! fclose(X_VAR[1])
   return .F.
endif
release X_VAR
release pf_VAR
release pf_VAL
return .T.

* ---------------
*  Function     : svGetString()
*  Parameters   : Variable Name
*  Return Value : The content of the variable as a string.
*  Notes        : If the variable is of type numeric, you would get 
*               : that number as a string
* ---------------

Function svGetString
param X_StrVar

if type("X_VAR[1]") == 'U'
   return ""         && The svOpen function is not called  !
endif

if (X_VAR[8] > 0)
   X_VAR[2] = 1
   do while X_VAR[2] <= X_VAR[8]
      if upper(X_StrVar) == upper(pf_VAR[X_VAR[2]])
	 return pf_VAL[X_VAR[2]]
      endif
      X_VAR[2] = X_VAR[2] + 1
   enddo
endif
return ""

* ---------------
*  Function     : svGetNum()
*  Parameters   : Variable Name
*  Return Value : The content of the variable as a number.
*  Notes        : If the variable is of type alpha numeric, you 
*               : would get the value as 0
* ---------------
Function svGetNum
param X_NumVar
return val(svGetString(X_NumVar))


* ---------------
*  Function     : svSetString()
*  Parameters   : Variable Name
*  Return Value : .T. if successful, .F. on failure.
*  Notes        : You can assign a new string value into a variable
* ---------------
Function svSetString
param X_StrVar, X_StrVal

if type("X_VAR[1]") == 'U'
   return .F.         && The svOpen function is not called  !
endif

if type("X_StrVal") != 'C'
   return .F.
endif

if X_VAR[8] > 0
   X_VAR[2] = 1
   do while X_VAR[2] <= X_VAR[8]
      if upper(X_StrVar) == upper(pf_VAR[X_VAR[2]])
	 pf_VAL[X_VAR[2]] = X_StrVal
	 return .T.
      endif
      X_VAR[2] = X_VAR[2] + 1
   enddo
endif

X_VAR[8] = X_VAR[8] + 1

if X_VAR[8] >= 200
   return .F.
endif

pf_VAR[X_VAR[8]] = X_StrVar
pf_VAL[X_VAR[8]] = X_StrVal
return .T.

* ---------------
*  Function     : svSetNum()
*  Parameters   : Variable Name
*  Return Value : .T. if successful, .F. on failure.
*  Notes        : You can assign a new numeric value into a variable
* ---------------
Function svSetNum
param X_NumVar, X_NumVal
if type("X_NumVal") != 'N'
   return .F.
endif
return svSetString(X_NumVar, alltrim(str(X_NumVal)))


* ---------------
*  Function     : Hex2Text()
*  Parameters   : Variable Name
*  Return Value : Translated String
*  Notes        : Translating Hex Charecters (For internal use Only)
* ---------------
Function Hex2Text
param X_NumVal

X_Tmpstr = ""
X_Tmpbyt = ""
X_Tmphx0 = 0
X_Tmphx1 = 0

for X_TmpCtr = 1 to len(X_NumVal)
   X_Tmpbyt = substr(X_NumVal,X_TmpCtr,1)
   if X_Tmpbyt == '%'
      if (X_TmpCtr+2) <= len(X_NumVal)
	 X_Tmphx1 = hexval(substr(X_NumVal,X_TmpCtr+1,1))
	 X_Tmphx0 = hexval(substr(X_NumVal,X_TmpCtr+2,1))
	 X_TmpCtr = X_TmpCtr + 2

	 X_Tmpbyt = chr(X_Tmphx0 + (X_Tmphx1 * 16))
      endif
   endif
   X_Tmpstr = X_Tmpstr + X_Tmpbyt
next
return X_Tmpstr


* ---------------
*  Function     : hexval()
*  Parameters   : Variable Name
*  Return Value : Translate char
*  Notes        : Translate hex byte to its value
* ---------------
function hexval
param byte

if byte >= '0' .and. byte <= '9'
   return (asc(byte) - 48)
endif

if byte >= 'a' .and. byte <= 'f'
   return (asc(byte) - 87)
endif

if byte >= 'A' .and. byte <= 'F'
   return (asc(byte) - 55)
endif

return 0



* ---------------
*  Function     : Text2Hex()
*  Parameters   : Variable Name
*  Return Value : Translated String
*  Notes        : Translating special characters into Hex Charecters (For internal use Only)
* ---------------
function Text2Hex
param X_NumVal

X_Tmpstr = ""
X_Tmpbyt = 0

for X_TmpCtr = 1 to len(X_NumVal)
   X_Tmpbyt = asc(substr(X_NumVal,X_TmpCtr,1))
   if X_Tmpbyt < 32 .or. X_Tmpbyt > 127
      X_Tmpstr = X_Tmpstr + "%" 
      X_Tmpstr = X_Tmpstr + hex(X_Tmpbyt)
   else
      X_Tmpstr = X_Tmpstr + chr(X_Tmpbyt)
   endif
next
return X_Tmpstr

* ---------------
*  Function     : Hex()
*  Parameters   : Byte
*  Return Value : Hex value
*  Notes        : (For internal use Only)
* ---------------
function Hex
param X_ParNum

X_TmpHx0 = int(X_ParNum / 16)
X_TmpHx1 = X_ParNum % 16

if X_TmpHx1 > 9
   X_hx1 = chr(55 + X_TmpHx1)
else
   X_hx1 = str(X_TmpHx1,0)
endif

if X_TmpHx0 > 9
   X_hx0 = chr(55 + X_TmpHx0)
else
   X_hx0 = str(X_TmpHx0,0)
endif

X_hex = alltrim(X_hx0)+alltrim(X_hx1)
return X_hex

